;; Mind Wandering task (CRT)
;; TODO: description of experiment

(defvar *response* nil)
(defvar *done*  nil)
(defvar *response-time* nil)
(defvar *allresp* nil)
(defvar *allrt* nil)
(defvar *stimuli* nil)
(defvar *start-time* nil)
(defvar *rt* nil)

;; ----------------------------------------------------------------
;; Define experiment (CRT)
;; Create a long array with strings of a series of digits
;; 1 block consists of: 2-5 random numbers, then a probe, then repeat until all 4 TPs and 16 targets are accounted for
;; 6 blocks total (3 CRT and 3 WM)
;; number 1-9: non-target
;; number 11-19: target (green) number
;; 99: thought probe

;; Random numbers in lisp:
;; (+ 2 (random 4)) returns number 2-5
;; (+ 1 (random 9)) returns number 1-9
;; (+ 11 (random 9)) returns number 11-19

;; Make list of 2-5 random numbers between 1-9 to put before targets
(defun randlist ()
  (let ((lst ()))
    (dotimes (i (+ 2 (random 4)))
       (setf lst (cons (+ 1 (random 9)) lst)))
   lst))
 

;; Make list of TPs and targets
(defun targetlist ()
  (let ((lst ()))
    (dotimes (i 16)
      (setf lst (cons (+ 11 (random 9)) lst)))
  lst))

;; Shuffle list
(defun shuffle (list)
  (let ((len (length list)))
    (loop repeat len
      do
        (rotatef
          (nth (random len) list)
          (nth (random len) list))
      finally
        (return list))))

;; Make long list of 2-5 randoms, then the first item from the targets and TPs list,
;; Then 2-5 again, then the second item from the list, repeat
(defun makecrtblock ()
  ;; Make list of targets and TPs (TP = 99) for the CRT task
  (setf targetstpscrt (concatenate 'list (targetlist) (make-array 4 :initial-element 99)))
  ;; Shuffle the targets and TPs so they are in random order
  (shuffle targetstpscrt)

  ;; Create a whole block
  (let ((lst ()))
    (dotimes (i 20)
      (setf lst (append (randlist) (cons (pop targetstpscrt) ()) lst)))
  lst))


;; Make a block for the WM task
;;use this for the WM model!
(defun makewmblock ()
  ;; Make list of targets (target "?" = 88) and TPs (TP = 99) for the WM task
  (setf targetstpswm (concatenate 'list (make-array 16 :initial-element 88) (make-array 4 :initial-element 99)))
  ;; Shuffle the targets and TPs so they are in random order
  (shuffle targetstpswm)
  
  ;; Create a whole block
  (let ((lst ()))
    (dotimes (i 20)
      (setf lst (append (randlist) (cons (pop targetstpswm) ()) lst)))
  lst))

;;Function to convert list of numbers to list of strings
(defun strlist (lst)
  (when lst
    (append (list (write-to-string (car lst))) (strlist (cdr lst)))))

;; ----------------------------------------------------------------

(defun do-exp (n)
;; runs a sequence of trials. n can be a positive number
  (reset)
  (setf *allresp* nil)
  (setf *allrt* nil)
  ;; make a long array with strings of 811 digits and 77 3s; Also include 24 thought probes
 ;; (setf *stimuli* (permute-list (concatenate 'list (make-array 811 :initial-element "d") (make-array 77 :initial-element "3"))))
;;  (setf *stimuli* (permute-list (concatenate 'list (make-array 405 :initial-element "d") (make-array 37 :initial-element "3") (make-array 12 :initial-element "Q"))))
 ;; for testing small stim set
 ; (setf *stimuli* (permute-list (concatenate 'list (make-array 2 :initial-element "d") (make-array 2 :initial-element "3") (make-array 2 :initial-element "Q"))))
;  (print *stimuli*)

  ;; CRT experiment: use makecrtblock function to make the list, use strlist to convert the list of numbers to a list of strings
  ;1 block: (setf *stimuli* (strlist (makecrtblock)))
  ;; All 3 blocks:
  (setf *crtblock* (concatenate 'list (makecrtblock) (makecrtblock) (makecrtblock) (makecrtblock) (makecrtblock) (makecrtblock)))
  (setf *stimuli* (strlist *crtblock*))

  
  (loop for str in *stimuli* do (do-trial str))

  ; print chunk activation
;  (print-chunk-activation attend-object)
;  (print-chunk-activation distracted)

  ; print the behavioral results
  (format t "mean ACC: ~a~%"  (perc *allresp* (reverse *stimuli*)))
  (format t "mean RT: ~a~%" (avg *allrt*))
  (format t "var RT: ~a~%" (var *allrt*))
  (format t "num incorrect omissions: ~a~%"  (num-incorrect-omissions *allresp* (reverse *stimuli*)))
)

(defun do-trial (str)
;; Is the current number green?
  (setf strgreen nil)
  ;; If str is between 11 and 19, subtract 10 from the number. The number is then no longer nil, so we know the number is green
  (if (> (parse-integer str) 10)
    (if (< (parse-integer str) 20)
      (setf strgreen (write-to-string (- (parse-integer str) 10)))))

;; Is it a thought probe? If so turn it into a "Q"
  (if (> (parse-integer str) 98)
    (setf str "Q"))

;; runs one trial. str can be either "3" or "d" (which is any other digit) or "Q", which is the thought probe.
  ;(reset) ;; do not reset, because we want the previous trial to affect the current
  (let 
    ((window (open-exp-window "CRT Experiment"                                      
                                 :visible nil
                                 :width 300
                                 :height 300
                                 :x 300
                                 :y 300)))
    
    ;; If the number is green, add green text, else add normal text.
    (if strgreen
      (add-text-to-exp-window :text strgreen
                              :color 'green
                              :width 30
                              :height 30
                              :x 145
                              :y 150)
      (add-text-to-exp-window :text str
                              :width 30
                              :height 30
                              :x 145
                              :y 150))
    (install-device window)
    (setf *response* nil)
    (setf *rt* nil)
    (setf *response-time* nil)
    (setf *start-time* (get-time))    
    ;(print *start-time*)
    (when *actr-enabled-p*
      (proc-display)
      (run 2)) ;;each trial is 2 seconds; stimulus presented for 500 ms
    (while (null *done*) 
      (allow-event-manager window))
    (clear-exp-window)
    (proc-display))

    ; if response time is not nil, then compute reaction time based on the start time of the trial
    (if *response-time* (setf *rt* (-  *response-time* *start-time*)))
    (print *rt*)
    ;(print *response-time*)
  (push *response* *allresp*)
  (push *rt* *allrt*)
)


(defmethod rpm-window-key-event-handler ((win rpm-window) key)
;; to catch the response
  (setf *response-time* (get-time))
  (setf *response* (string key))
  (setf *done* t)
)

;;;; analysis tools ;;;;
(defun get-expected-responses (stims)
        (loop for elem-index from 0 below (length stims) do 
              
              (let ((current-elem (nth elem-index stims)))
                     (cond
                         ((string= current-elem "1")
                             (setf (nth elem-index stims) nil)
                         )
                         ((string= current-elem "2")
                             (setf (nth elem-index stims) nil)
                         )
                         ((string= current-elem "3")
                             (setf (nth elem-index stims) nil)
                         )
                         ((string= current-elem "4")
                             (setf (nth elem-index stims) nil)
                         )
                         ((string= current-elem "5")
                             (setf (nth elem-index stims) nil)
                         )
                         ((string= current-elem "6")
                             (setf (nth elem-index stims) nil)
                         )
                         ((string= current-elem "7")
                             (setf (nth elem-index stims) nil)
                         )
                         ((string= current-elem "8")
                             (setf (nth elem-index stims) nil)
                         )
                         ((string= current-elem "9")
                             (setf (nth elem-index stims) nil)
                         )
                         
                         ((string= current-elem "11")
                             (setf (nth elem-index stims) "o")
                         )
                         ((string= current-elem "12")
                             (setf (nth elem-index stims) "e")
                         )
                         ((string= current-elem "13")
                             (setf (nth elem-index stims) "o")
                         )
                         ((string= current-elem "14")
                             (setf (nth elem-index stims) "e")
                         )
                         ((string= current-elem "15")
                             (setf (nth elem-index stims) "o")
                         )
                         ((string= current-elem "16")
                             (setf (nth elem-index stims) "e")
                         )
                         ((string= current-elem "17")
                             (setf (nth elem-index stims) "o")
                         )
                         ((string= current-elem "18")
                             (setf (nth elem-index stims) "e")
                         )
                         ((string= current-elem "19")
                             (setf (nth elem-index stims) "o")
                         )
                         ((string= current-elem "99")
                             (setf (nth elem-index stims) "Q")
                         )
                         ;((string= current-elem "3")
                         ;    (setf (nth elem-index stims) nil)
                         ;)
                         ;((string= current-elem "Q")
                            ; (setf (nth elem-index stims) "Q")
                         ;)
                     )
                )
        )
        stims
)

(defun perc (resp stim)
;; check for how many instances the response nil is given to 3 and the response "f" is given to any other digit "d"
  (let ((s 0.0) 
        (correctresp (get-expected-responses stim)))
    (dolist (i resp)
      (when (equalp i (pop correctresp)) (incf s 1)))
    (/ s (length resp))))

(defun num-incorrect-omissions (resp stim)
;; add up the number of omission trials
  (let ((s 0.0) 
        (correctresp (get-expected-responses stim)))
    (print correctresp)
    (print resp)
    (dolist (i resp)
      (when (and (not (equalp i "99")) (not (equalp i (pop correctresp)))) (incf s 1)))
    s)
)

(defun avg (numbers)
  (setf numbersnotnil (remove nil numbers))
  (if (null numbersnotnil)
      (error "Average of the empty list is undefined.")
      (float (/ (reduce #'+ numbersnotnil)
         (length numbersnotnil)))))


(defmacro square (x)
  `(* ,x ,x))

(defun var (numbers)
  (setf numbersnotnil (remove nil numbers))
  (print numbers)
  (let ((mean (avg numbersnotnil))
        (n (length numbersnotnil)))
    (/ (reduce #'+ (map 'list #'(lambda (x) (square (- mean x))) numbersnotnil))
       (1- n))))


(clear-all)

(define-model mindwandering

  ; behaviorBSnew1
  ; behaviorBSslow1
  ; behaviorBSmoreVar1
;(sgp :v  "/Dropbox/School Stuff/- 2014 2015/Bachelorproject/mindwanderingModels/Nieuw/BSnoLearnPump5.txt" ;t ; tracing verbose
(sgp :v  "/Users/Joram/Dropbox/meditationModel/Model/data/BSnoLearnPumpCRT.txt" ;t ; tracing verbose 
     :rt -3 ; retrieval threshold
     :trace-detail low 
     :save-buffer-trace t
     :sact t ; save activation values
     :imaginal-action-activation 1.0 ; make sure that imaginal-action also causes spreading activation
   ;  :visual-location-activation 1.0 ;this makes sure that any appearance of info in the visual field can nudge the participant back to attending
     :show-focus t
     :act low ;t/medium/low/nil ;;tracing activation values
     :er t ;;;if er=t then ties will be broken randomly; if er=nil then always the same production will be chosen with ties
     :mas 2 ;; mas: activate spreading activation
     :esc t
     :declarative-num-finsts 4 ;; :declarative-num-finsts: number of items that are kept as recently retrieved (default 4)
     :declarative-finst-span 3 ;; :declarative-finst-span: how long items stay in the recently-retrieved state (default 3)
     :lf 1 ; latency factor used to turn activation values into retrieval times
     :ans 0.13 ;activation noise turned from 0.2 to .15 on 28 mar; 0.12 on 15 may
     :visual-attention-latency 0
     :ul t ;; sgp :ul t -> now the program can learn utilities when certain productions are associated with a reward; In meditation, paying attention for a certain amount of time should lead to a reward. 
     :egs 0.1 ; :egs expected gain in utility: noise in utility computation
     :model-warnings nil 
     :bll 0.5 ;; :bll = set base-level learning, such that items more frequently/recently retrieved gain higher activation
     ; parameters for base-level inhibition
     :enable-inhibition nil ; turn on the base-level inhibition module
     :inhibition-scale 10 ; 10 is best from Lebiere & Best
     :inhibition-decay 1 ; 1 is best from Lebiere & Best
     :ol nil  ; set optimized learning to nil to ensure that base-level inhibition works
)

(chunk-type goal state)
(chunk-type lemma value)
(chunk-type concept value)
(chunk-type srmapping value hand oddeven)
(chunk-type observing state upper-y lower-y)
(chunk-type object name medobject)
(chunk-type item name valence volition)
(chunk-type distracted state)
(chunk-type beginning state)
(chunk-type see-thoughts state)
(chunk-type planning state)
(chunk-type subgoal state)

(add-dm (lemma isa chunk)
        (left isa chunk)
        (right isa chunk)
        (n1 isa lemma value "1")
        (n2 isa lemma value "2")
        (n3 isa lemma value "3")
        (n4 isa lemma value "4")
        (n5 isa lemma value "5")
        (n6 isa lemma value "6")
        (n7 isa lemma value "7")
        (n8 isa lemma value "8")
        (n9 isa lemma value "9")
        (thoughtProbe isa lemma value "Q")
        (n1press isa srmapping value "1" hand right oddeven "o")
        (n2press isa srmapping value "2" hand left oddeven "e")
        (n3press isa srmapping value "3" hand right oddeven "o")
        (n4press isa srmapping value "4" hand left oddeven "e")
        (n5press isa srmapping value "5" hand right oddeven "o")
        (n6press isa srmapping value "6" hand left oddeven "e")
        (n7press isa srmapping value "7" hand right oddeven "o")
        (n8press isa srmapping value "8" hand left oddeven "e")
        (n9press isa srmapping value "9" hand right oddeven "o")
        (OntaskPress isa srmapping value "Q" hand left)
	(attending isa chunk)
	(yes isa chunk)
	(no isa chunk)
	(pos isa chunk)
	(neg isa chunk)
	(neut isa chunk)
	(approach isa chunk)
	(avoid isa chunk)
	(stay isa chunk)
	(attention-object isa object name "MEDOBJECT" medobject yes)
        
	;; populate the thought pump
	(itemposA ISA item name "posA" valence pos volition stay)
	(itemposS ISA item name "posS" valence pos volition stay)
	(item1 ISA item name "1" valence pos volition stay)
	(item2 ISA item name "2" valence pos volition stay)
	(item3 ISA item name "3" valence pos volition stay)
	;(item4 ISA item name "4" valence pos volition stay)
	;(item5 ISA item name "5" valence pos volition stay)
	(item6 ISA item name "6" valence pos volition approach)
	(item7 ISA item name "7" valence pos volition approach)
	(item8 ISA item name "8" valence pos volition approach)
	(item9 ISA item name "9" valence pos volition approach)
	(item10 ISA item name "10" valence pos volition avoid)
	;(item11 ISA item name "11" valence pos volition avoid)
	;(item12 ISA item name "12" valence pos volition avoid)

	(item13 ISA item name "13" valence neg volition avoid)
	(item14 ISA item name "14" valence neg volition stay)
	(itemnegV ISA item name "negV" valence neg volition avoid)
	(itemnegS ISA item name "negS" valence neg volition stay)
	(item15 ISA item name "15" valence neg volition avoid)
	(item16 ISA item name "16" valence neg volition avoid)
	(item17 ISA item name "17" valence neg volition avoid)
	(item18 ISA item name "18" valence neg volition stay)
	(item19 ISA item name "19" valence neg volition stay)
	;(item20 ISA item name "20" valence neg volition stay)
	;(item21 ISA item name "21" valence neg volition stay)
	(item22 ISA item name "22" valence neg volition approach)
	;(item23 ISA item name "23" valence neg volition approach)
	;(item24 ISA item name "24" valence neg volition approach)

	(itemneuA ISA item name "neuA" valence neut volition approach)
	(itemneuV ISA item name "neuV" valence neut volition avoid)
	(itemneuS ISA item name "neuS" valence neut volition stay)
	(item25 ISA item name "25" valence neut volition stay)
	(item26 ISA item name "26" valence neut volition stay)
	(item27 ISA item name "27" valence neut volition stay)
	(item28 ISA item name "28" valence neut volition stay)
	(item29 ISA item name "29" valence neut volition stay)
	(item30 ISA item name "30" valence neut volition approach)
	;(item31 ISA item name "31" valence neut volition approach)
	;(item32 ISA item name "32" valence neut volition approach)
	(item33 ISA item name "33" valence neut volition avoid)
	(item34 ISA item name "34" valence neut volition avoid)
	;(item35 ISA item name "35" valence neut volition avoid)
	;(item36 ISA item name "36" valence neut volition stay)

	(itemAttend ISA item name "attend" valence pos volition stay)

	(attend-object isa goal state attend-to-object )
	(distracted isa goal state get-distracted )
       ; (goal isa distracted state remember-items)
        ;(startgoal isa goal state start)
        (startgoal isa beginning state start)
	(incgoal isa planning state strengthen-goal )
) 


(set-base-levels 
 (attend-object 600 -10000)
 (distracted 200 -10000)
 (n1 10000 -1000000)
 (n2 10000 -1000000)
 (n3 10000 -1000000)
 (n4 10000 -1000000)
 (n5 10000 -1000000)
 (n6 10000 -1000000)
 (n7 10000 -1000000)
 (n8 10000 -1000000)
 (n9 10000 -1000000)
 (thoughtprobe 10000 -1000000)
 (n1press 10000 -1000000)
 (n2press 10000 -1000000)
 (n3press 10000 -1000000)
 (n4press 10000 -1000000)
 (n5press 10000 -1000000)
 (n6press 10000 -1000000)
 (n7press 10000 -1000000)
 (n8press 10000 -1000000)
 (n9press 10000 -1000000)
 (onTaskPress 10000 -1000000)
 (itemposA 200 -100000)
 (itemposS 200 -100000)
 (itemnegV 200 -100000)
 (itemnegS 200 -100000)
 (itemneuA 200 -100000)
 (itemneuV 200 -100000)
 (itemneuS 200 -100000)
 (item1 200 -100000)  
 (item2 200 -100000)  (item13 200 -100000)  
 (item3 200 -100000)  (item14 200 -100000)  (item25 200 -100000)
  (item15 200 -100000)  (item26 200 -100000)
  (item16 200 -100000)  (item27 200 -100000)
 (item6 200 -100000)  (item17 200 -100000)  (item28 200 -100000)
 (item7 200 -100000)  (item18 200 -100000)  (item29 200 -100000)
 (item8 200 -100000)  (item19 200 -100000)  (item30 200 -100000)
 (item9 200 -100000)   
 (item10 200 -100000)   
  (item22 200 -100000)  (item33 200 -100000)
 (item34 200 -100000) 
 (itemAttend 220 -100000)
)

;; start the task by retrieving the attend goal and strengthening it
(P intend-to-attend-goal
   =goal>
     isa beginning
     state start
  ?retrieval>
     buffer    empty
     state     free
   - state   error
==>
  +retrieval>
    isa goal
    state attend-to-object
  -goal>
)



; retrieve the current goal (done many times while paying attention, until the visual location buffer is stuffed); the funny retrieval syntax in the action part of the production is intended to prevent that the goal is not strengthened during the retrieval
(p check-current-goal
  ?retrieval>
     state   free
   - state   error
   =retrieval>
     isa goal
     state =statename
   - state get-distracted
   ?goal>
     buffer empty
   ?visual>
   - scene-change T
==>   
  =retrieval>
    state nil
  -retrieval>
  +retrieval>
    isa goal
  - state nil
)

; attend to a stimulus when there has just been a scene change
(P attend-stim
   =retrieval>
      isa         goal
      state       attend-to-object
  ?retrieval>
     state     free
   - state     error
   ?visual>
      state       free
      scene-change T
   ?goal>
      buffer empty
==>
   +goal>
      isa subgoal
      state       identify
   +visual-location>
      isa         visual-location
;      :attended   nil
      screen-x    lowest
      screen-y    lowest
   ; this part ensures that the attend-object goal does not get strengthened when it gets flushed
   =retrieval> 
     state nil
   -retrieval>
)

(P identify-stim
   ;; =retrieval>
   ;;    isa         goal
   ;;    state       attend-to-object
   =goal>
      isa         subgoal
      state       identify
   =visual-location>
      isa         visual-location
   ?visual>
      state       free
==>
   +visual>
      isa         move-attention
      screen-pos  =visual-location
   +goal>
   isa           subgoal
   state         check-S-R
   ;; =retrieval>
   ;;   state nil
   ;; -retrieval>
   ;; +retrieval>
   ;;   isa goal
   ;;   state attend-to-object
)

; check-whether-O
(P check-whether-O
   =visual>
      isa         text
      value       =val
      color       =col
   =goal>
      isa         subgoal
      state       check-S-R
   ;; =retrieval>
   ;;    isa         goal
   ;;    state       attend-to-object
   ?retrieval>
     state   free
   - state   error
==>
  ;; =retrieval>
  ;;    state nil
  ;; -retrieval>  
 +retrieval>
      isa         lemma
      value       =val
  +goal>
     isa subgoal
     state get-S-R-result
  +visual>
      isa clear-scene-change
)

(P get-result-of-check
  =retrieval>
      isa         lemma
      value       =val
  =visual-location>
      isa         visual-location
      color       =col
  =goal>
      isa subgoal
      state get-S-R-result
==>
   ;=imaginal-action> =retrieval
   +retrieval>
      isa         srmapping
      value       =val
   +visual-location>
    isa visual-location
    color =col
   +goal>
      isa subgoal
      state do-response
)

; implement response if "even" or "odd"; then retrieve a new goal
(P respond
   ?manual>
      state       free
   =visual-location>
      isa visual-location
      color GREEN
   =retrieval>
      isa srmapping
      value       =val
      oddeven     =oddeven
     - value      "Q" 
      hand        =hand
   =goal>
      isa subgoal
      state do-response
==>
   ;-imaginal-action>
   +manual>
      isa         press-key
      key         =oddeven
      !output!    =oddeven
   -goal>
   -visual-location>
   -visual>
   +retrieval>
      isa goal
   -  state nil
)


; implement response to a thought probe when paying attention: "on-task"
(P respond-on-task
   ?manual>
      state       free
   =retrieval>
      isa srmapping
      value       "Q"
      hand        =hand
   =goal>
      isa subgoal
      state do-response
==>
   ;-imaginal-action>
   +manual>
      isa         press-key
      key         "Q"
      !output!    "Q"
   -goal>
   -visual-location>
   -visual>
   +retrieval>
      isa goal
   -  state nil
)


; implement no-response if "BLACK"
(P do-not-respond
   =visual-location>
      isa visual-location
      color BLACK
   =goal>
      isa subgoal
      state do-response
==>
   -goal>
   -visual-location>
   -visual>
   +retrieval>
     isa goal
   - state nil
)

; respond the default when you have not retrieved SR mapping and when you are distracted and something appears in the visual-location buffer; also make sure that you are not busy retrieving when responding with default
(P respond-default
   ?manual>
      state       free
   ?visual-location>
   - buffer empty
   =goal>
      isa distracted
   ?visual>
      scene-change T
      state free
   ?retrieval>
      state free
   -  state error
==>
   +manual>
      isa         press-key
      key         "e"
      !output!    "e"
   =goal>
   -visual-location>
   -visual>
)

;; PRODUCTIONS FOR THE THOUGHT PUMP THAT REFLECTS THE DISTRACTION
(p start-thought-pump
 =retrieval>
   isa goal
   state get-distracted
==>
  +goal>
    isa distracted
    state remember-items
  +retrieval>
    isa item
  - volition nil
    :recently-retrieved nil
 -imaginal>
)


;; continue thought pump by retrieving subsequent memories
;; follow-up memories occur when the volition is approach or avoid; when volition is stay, then return to meditation object
(p recall-next
   =goal>
     isa distracted
     state remember-items
   =retrieval>
     isa item
     name =itemname
   - name "attend"
     valence =val
     volition =vol
   ?retrieval>
     state free
   - state error
==>
   ; change the current memory so that it does not increase in activation
   =retrieval>
     volition nil
  ; move the current memory in the imaginal-action, from where it can spread activation
  =imaginal-action> =retrieval
  ; retrieve the next item, which is different from the one recently recalled
  +retrieval>
     isa item
     - name =itemname
     - volition nil
     :recently-retrieved nil
  ; only ask for items that were not recently retrieved to prevent one item from dominating the model
  ;=imaginal>  
  =goal>
)




;; if the remembered item is the attention object, return to pay-attention (with some probability), which brings you back to the task
(p remember-to-attend
   =goal>
     isa distracted
     state remember-items
   ?retrieval>
     state free
   =retrieval>
     isa item
     name "attend"
     valence =val
     volition =vol
==>
  ; put in the imaginal module-> spotlight of attn 
  ; then get ready to retrieve the attending goal
  ;; +imaginal>
  ;;   isa item
  ;;   name "attend"
  ;;   valence =val
  ;;   volition =vol
  =retrieval>
   name "done"
  -imaginal-action>
  +goal>
    isa beginning
    state start
  -retrieval>
)

(goal-focus startgoal)

(setf *actr-enabled-p* t)

; add connection weights, which make it more likely to remember a positive item once a positive item has been retrieved, and vice versa.
(add-sji (distracted itemnegV 2) (distracted itemneuV 2)
  (lemma itemattend 0.5) (lemma attend-object 0.1)
  ;(hand itemattend 2) (hand attend-object 2)
)
; to encourage the thought pump to end, we make it so that some retrieved memories prime the "attending" chunk
(add-sji (pos itemattend .2)  (stay itemattend .2) (neg itemattend .1) (neut itemattend .1) (approach itemattend .1) (avoid itemattend .1))

;(add-sji (attend-object item13 2) (distracted itemnegV 2) (distracted itemneuV 2))
;; (add-sji (approach approach 2) (avoid avoid 2) (stay stay 2) (stay approach 1) (approach stay 1) (stay avoid 0) (avoid stay 0) (approach avoid 0))
;; ; connection between meditation-object and "meditate"
;; (add-sji (meditation-object item13 2))
;; (add-sji (pos pos 2) (pos neg 0) (neg pos 0) (neg neg 2))
;; ; between positive items
;; (add-sji (item1 item2 2)  (item1 item3 2)  (item1 item4 2)  (item2 item3 2)  (item2 item4 2)  (item3 item4 2) (item1 item13 2) (item2 item13 2) (item3 item13 2) (item13 item4 2))
;; (add-sji (item2 item1 2)  (item3 item1 2)  (item4 item1 2)  (item3 item2 2)  (item4 item2 2)  (item4 item3 2))
;; ;between negative items
;; (add-sji (item5 item6 2)  (item5 item7 2)  (item5 item8 2)  (item6 item7 2)  (item6 item8 2)  (item7 item8 2))
;; (add-sji (item6 item5 2)  (item7 item5 2)  (item8 item5 2)  (item7 item6 2)  (item8 item6 2)  (item8 item7 2))

; set utility of responding default such that it is likely to fire
(spp respond-default :u 0.5)


)


